<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Domain\Model;

use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
use IMIA\BaseExt\Annotation\SQL;
use IMIA\BaseExt\Annotation\TCA;

/**
 * @package     imia_pageteaser
 * @subpackage  Domain\Model
 * @author      David Frerich <d.frerich@imia.de>
 *
 * @TCA\Table(
 *  types = {
 *      100: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      103: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.external;external, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      104: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcut;shortcut, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcutpage;shortcutpage, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      110: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      111: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      112: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.external;external, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      113: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.external;external, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      114: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcut;shortcut, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcutpage;shortcutpage, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      120: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      123: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.external;external, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      124: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcut;shortcut, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcutpage;shortcutpage, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      130: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      133: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.external;external, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      134: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcut;shortcut, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcutpage;shortcutpage, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      140: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      143: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.external;external, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      144: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcut;shortcut, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcutpage;shortcutpage, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;hiddenonly, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media"
 *  }
 * )
 */
class PageAlt extends AbstractPageAlt
{
    /**
     * @SQL\Column(name="pid", create=false)
     * @var \IMIA\ImiaPageteaser\Domain\Model\Page
     * @lazy
     */
    protected $page;

    /**
     * @TCA\Select(
     *  items = {
     *      99: {"LLL::page_doktype_divider_news", "--div--"},
     *      100: {"LLL::page_doktype_news", 100, "EXTREL::Resources/Public/Images/Icons/PageDoktypeNews.png"},
     *      101: {"LLL::page_doktype_news_shortcut", 104, "EXTREL::Resources/Public/Images/Icons/PageDoktypeNewsShortcut.png"},
     *      102: {"LLL::page_doktype_news_external", 103, "EXTREL::Resources/Public/Images/Icons/PageDoktypeNewsExternal.png"},
     *      109: {"LLL::page_doktype_divider_gallery", "--div--"},
     *      110: {"LLL::page_doktype_gallery", 110, "EXTREL::Resources/Public/Images/Icons/PageDoktypeGallery.png"},
     *      112: {"LLL::page_doktype_gallery_shortcut", 114, "EXTREL::Resources/Public/Images/Icons/PageDoktypeGalleryShortcut.png"},
     *      113: {"LLL::page_doktype_gallery_external", 113, "EXTREL::Resources/Public/Images/Icons/PageDoktypeGalleryExternal.png"},
     *      114: {"LLL::page_doktype_gallery_images", 111, "EXTREL::Resources/Public/Images/Icons/PageDoktypeGalleryImages.png"},
     *      115: {"LLL::page_doktype_gallery_video", 112, "EXTREL::Resources/Public/Images/Icons/PageDoktypeGalleryVideo.png"},
     *      119: {"LLL::page_doktype_divider_product", "--div--"},
     *      120: {"LLL::page_doktype_product", 120, "EXTREL::Resources/Public/Images/Icons/PageDoktypeProduct.png"},
     *      122: {"LLL::page_doktype_product_shortcut", 124, "EXTREL::Resources/Public/Images/Icons/PageDoktypeProductShortcut.png"},
     *      123: {"LLL::page_doktype_product_external", 123, "EXTREL::Resources/Public/Images/Icons/PageDoktypeProductExternal.png"},
     *      129: {"LLL::page_doktype_divider_event", "--div--"},
     *      130: {"LLL::page_doktype_event", 130, "EXTREL::Resources/Public/Images/Icons/PageDoktypeEvent.png"},
     *      132: {"LLL::page_doktype_event_shortcut", 134, "EXTREL::Resources/Public/Images/Icons/PageDoktypeEventShortcut.png"},
     *      133: {"LLL::page_doktype_event_external", 133, "EXTREL::Resources/Public/Images/Icons/PageDoktypeEventExternal.png"},
     *      139: {"LLL::page_doktype_divider_article", "--div--"},
     *      140: {"LLL::page_doktype_article", 140, "EXTREL::Resources/Public/Images/Icons/PageDoktypeArticle.png"},
     *      142: {"LLL::page_doktype_article_shortcut", 144, "EXTREL::Resources/Public/Images/Icons/PageDoktypeArticleShortcut.png"},
     *      143: {"LLL::page_doktype_article_external", 143, "EXTREL::Resources/Public/Images/Icons/PageDoktypeArticleExternal.png"}
     *  }
     * )
     *
     * @var integer
     */
    protected $doktype;

    /**
     * @SQL\Column(type="int", length=11, nullable=false)
     * @TCA\ImageFiles(
     *  l10n_mode = "mergeIfNotBlank",
     *  maxitems = 1,
     *  foreign_types = {
     *      \TYPO3\CMS\Core\Resource\File::FILETYPE_IMAGE: {"showitem": "title,alternative;;;;3-3-3,--palette--;;filePalette" }
     *  }
     * )
     * @TCA\AddToTypes(position="after:abstract")
     *
     * @var integer
     */
    protected $teaserImage;

    /**
     * @SQL\Column(type="int", length=11, nullable=false)
     * @TCA\ImageFiles(
     *  foreign_types = {
     *      \TYPO3\CMS\Core\Resource\File::FILETYPE_IMAGE: {"showitem": "title,alternative;;;;3-3-3,description,--palette--;;filePalette" }
     *  }
     * )
     *
     * @var integer
     */
    protected $galleryImages;

    /**
     * @return \IMIA\ImiaPageteaser\Domain\Model\Page
     */
    public function getPage()
    {
        return $this->page;
    }

    /**
     * @param \IMIA\ImiaPageteaser\Domain\Model\Page $page
     * @return \IMIA\ImiaPageteaser\Domain\Model\PageAlt
     */
    public function setPage(\IMIA\ImiaPageteaser\Domain\Model\Page $page)
    {
        $this->page = $page;

        return $this;
    }
}